/**
 * Title:        BarcodeXml.java
 * Description:  Sample code for creating barcode images based on the XML API
 * Copyright:    Copyright (c) 2013
 * Company:      Hyfinity Ltd
 */
package com.hyfinity.barcode.samples;

import org.w3c.dom.Node;
import java.io.InputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.awt.image.BufferedImage;
import org.apache.commons.codec.binary.Base64;

import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.Result;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.TransformerFactory;

import org.krysalis.barcode4j.BarcodeGenerator;
import org.krysalis.barcode4j.BarcodeUtil;
import org.krysalis.barcode4j.output.bitmap.BitmapCanvasProvider;

import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.DefaultConfigurationBuilder;

import com.hyfinity.utils.xml.XDocument;

public class BarcodeXml
{
    /**
     * Generate a base64 encoded barcode image string from an xml fragment containing the necessary barcode info.
     * @param barcodeElem XML fragment containing the barcode info, according to the Barcode4J structure. (http://barcode4j.sourceforge.net/2.1/barcode-xml.html)
     * @return base64 encoded barcode image string
     */
    public static String generateBarcode(Node barcodeElem) throws Exception
    {
        //System.out.println("barcode element="+new XDocument(barcodeElem));
        String msg = new XDocument(barcodeElem).selectString("/barcode/@message");
        //System.out.println("barcode message="+msg);

        //Need an InputStream for the DefaultConfigurationBuilder
        ByteArrayOutputStream outStream = new ByteArrayOutputStream();
        Source xmlSource = new DOMSource(barcodeElem);
        Result outTarget = new StreamResult(outStream);
        TransformerFactory.newInstance().newTransformer().transform(xmlSource, outTarget);
        InputStream inStream = new ByteArrayInputStream(outStream.toByteArray());

        DefaultConfigurationBuilder builder = new DefaultConfigurationBuilder();
        Configuration cfg = builder.build(inStream);

        BarcodeGenerator gen = BarcodeUtil.getInstance().createBarcodeGenerator(cfg);

        ByteArrayOutputStream out = new ByteArrayOutputStream();
        BitmapCanvasProvider provider = new BitmapCanvasProvider(
            out, "image/x-png", 300, BufferedImage.TYPE_BYTE_GRAY, true, 0);
        gen.generateBarcode(provider, msg);

        provider.finish();
        out.close();

        return "data:image/png;base64," + new String(Base64.encodeBase64(out.toByteArray()));
    }
}

